﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;
using System.ComponentModel;
using DomainModel;
using IdeaBlade.EntityModel;
using System.Windows;
using System.Collections.Specialized;

namespace WPFTelerikWalk {
  class MainWindowViewModel : INotifyPropertyChanged {

    public MainWindowViewModel() {
      _mgr.EntityServerError += new EventHandler<EntityServerErrorEventArgs>(_mgr_EntityServerError);

      Products = new ObservableCollection<Product>();
      Categories = new ObservableCollection<Category>();
      Suppliers = new ObservableCollection<Supplier>();
      Messages = new ObservableCollection<string>();

      Log("Starting...");

      foreach (Product product in _mgr.Products) {
        Products.Add(product);
      }
      CurrentProduct = Products.First();

      foreach (Category category in _mgr.Categories) {
        Categories.Add(category);
      }

      foreach (Supplier supplier in _mgr.Suppliers) {
        Suppliers.Add(supplier);
      }
    }

    public void SaveChanges() {
      Log("Saving...");
      SaveResult saveResult;
      try {
        saveResult = _mgr.SaveChanges();
        if (saveResult.SaveStatus == SaveStatus.Normal) {
          Log("Successfully saved " + saveResult.SavedEntities.Count().ToString() + " entities.");
        } else if (saveResult.SaveStatus == SaveStatus.NoOperation) {
          Log("No entities to be saved.");
        } else {
          Log("Save failed.");
        }
      } catch (Exception exception) {
        Log("Save failed: " + exception.Message);
      }
    }

    private void Log(string message) {
      message = String.Format("{0}: {1}", ++_messageCounter, message);
      Messages.Insert(0, message); // new messages on top
      //logListBox.SelectedItem = message;
      //logListBox.SelectedItem = null;
    }

    private void _mgr_EntityServerError(object sender, EntityServerErrorEventArgs e) {
      MessageBox.Show(e.Exception.Message);
    }

    public event PropertyChangedEventHandler PropertyChanged = delegate { };

    private void RaisePropertyChanged(string property) {
      PropertyChanged(this, new PropertyChangedEventArgs(property));
    }

    private int _messageCounter;
    public ObservableCollection<Product> Products { get; set; }
    public ObservableCollection<Category> Categories { get; set; }
    public ObservableCollection<Supplier> Suppliers { get; set; }
    public ObservableCollection<string> Messages { get; set; }
    private NorthwindIBEntities _mgr = new NorthwindIBEntities();
    private Product _currentProduct;
    public Product CurrentProduct {
      get { return _currentProduct; }
      set { _currentProduct = value; RaisePropertyChanged("CurrentProduct"); }
    }
  }
}
